//+------------------------------------------------------------------+
//|                                                    MA ribbon.mq4 |
//|                                               mladenfx@gmail.com |
//|                                                                  |
//| original idea by Jose Silva                                      |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"
// modification kinonen Fev 2009 & 2010
//
// T3MA_ribbon_multi.mq4 (padnoter)
// modified padnoter May 2014. Allows ribbons in levels as per "slowkey"
//    level0 is always current tf. level1 is the next MT4 tf up etc.
//    means 7 indis will do current tf plus 5 levels up plus a shadow down... 
//    which will also work as you change TF (no need for more indi & visualisation off)
// 6/may/14 padnoter. add HistoryAlert/HistoryComment to alert if not enough bars to calc ribbons
// 6/may/14 padnoter. add alt. colour option (ribbonColoursConstant), add index labels, 
//                    add chart ribbon labels (showRibbonLabels)

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 DarkSlateGray
#property indicator_color2 DarkSlateGray
#property indicator_color3 White
#property indicator_color4 Crimson
#property indicator_width1 5
#property indicator_width2 5
#property indicator_width3 3
#property indicator_width4 3

//
//
extern int       MA1Period=36;
extern double    factor1=0.54;
extern int       MA2Period =54;
extern double    factor2=0.63;
extern int       levelsUP=0;
extern color     level0color=DarkSlateGray;
extern color     level1color=Navy;
extern color     level2color=DarkGreen;
extern color     level3color=Brown;
extern color     level4color=C'48,87,120'; //=DarkSlateBlue
extern color     level5color=C'27,44,44'; //=screen door

extern int       TimeFrame =0;
extern int       NbOfCandles=500;
extern bool      HistoryAlert=False;
extern bool      HistoryComment=True;
extern string    t1="if ribbonColoursConstant=True then level0color is 5m, level1color is 15m, level2color is 60m etc";
extern bool      ribbonColoursConstant=true; // Just type false to go back to changing colors. 
extern string    t2="if showRibbonLabels=True then labels will be added on right-h-s to each ribbon";
extern bool      showRibbonLabels=false; // Type true to turn on all the time. 
extern string    labelFont = "Arial";
extern int       labelFontSize=10; // Type the size you like and compile above. If you change it back compile it again. 
extern color     FontColour=White;
//
color  ribbonColor=level0color;
color  fastColor=White;
bool   screenDoor=False;
string ribbonlabel;
string prefix="t3rm_";
double buffer1[];
double buffer2[];
double buffer3[];
double buffer4[];

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//

int init()
{
   int timeframe=Period(); // minutes
   int nextTF=timeframe;
   if (levelsUP>=1)
   {
      nextTF=getNextTF(timeframe);
      ribbonColor=level1color;
   }
   if (levelsUP>=2)
   {
      nextTF=getNextTF(nextTF);
      ribbonColor=level2color;
   }
   if (levelsUP>=3)
   {
      nextTF=getNextTF(nextTF);
      ribbonColor=level3color;
   }
   if (levelsUP>=4)
   {
      nextTF=getNextTF(nextTF);
      ribbonColor=level4color;
   }
   if (levelsUP>=5)
   {
      nextTF=getNextTF(nextTF);
      ribbonColor=level5color;
   }


   // override ribbon colour with constant colour IF param set   
   if (ribbonColoursConstant)
      setConstantColour(nextTF);

   if (levelsUP==-1) //shadow
   {
      nextTF=getPrevTF(timeframe);
      ribbonlabel=getNtoTF(nextTF)+" shadow ribbon";
   }
   MA1Period=MA1Period*nextTF/timeframe;
   MA2Period=MA2Period*nextTF/timeframe;
   if (MA2Period>Bars) // NOT ENOUGH BARS IN HISTORY TO SHOW CORRECTLY
   {
      if (HistoryAlert)
         Alert(Symbol()+" "+Period()+" - Not enough historical bars "+Bars+" (need "+MA2Period+") to calc "+levelsUP+" levelsUP = "+nextTF);
      if (HistoryComment)
         Comment("Low history, some ribbons missing, see .Terminal>Experts tab");
      // always print in terminal>experts tab
      Print("Not enough historical bars "+Bars+" (need "+MA2Period+") to calc "+levelsUP+" levelsUP = "+nextTF);
      MA1Period=0;
      return(0);
   }

   if (levelsUP>=0)
   {
      ribbonlabel=getNtoTF(nextTF)+" ribbon(+"+(string)levelsUP+")";
      SetIndexBuffer(0,buffer3); SetIndexStyle(0,DRAW_HISTOGRAM, STYLE_SOLID, 5, ribbonColor);
      SetIndexBuffer(1,buffer4); SetIndexStyle(1,DRAW_HISTOGRAM, STYLE_SOLID, 5, ribbonColor);
      SetIndexStyle(2,DRAW_LINE, STYLE_SOLID, 3, fastColor);
      if (screenDoor) // W1
      {
         ribbonColor=level5color;
         SetIndexBuffer(0,buffer3); SetIndexStyle(0,DRAW_HISTOGRAM, STYLE_DOT, 1, ribbonColor);
         SetIndexBuffer(1,buffer4); SetIndexStyle(1,DRAW_HISTOGRAM, STYLE_DOT, 1, ribbonColor);
      }
   }
   else // shadow
   {
      SetIndexBuffer(0,buffer3); SetIndexStyle(0,DRAW_HISTOGRAM, STYLE_SOLID, 1, CLR_NONE);
      SetIndexBuffer(1,buffer4); SetIndexStyle(1,DRAW_HISTOGRAM, STYLE_SOLID, 1, CLR_NONE);
      SetIndexStyle(2,DRAW_LINE, STYLE_SOLID, 3, PaleGoldenrod);
   }
   SetIndexBuffer(2,buffer1);
   SetIndexBuffer(3,buffer2);
   SetIndexLabel(0,ribbonlabel);
   SetIndexLabel(1,ribbonlabel);
   SetIndexLabel(2,ribbonlabel+"(fast)");
   SetIndexLabel(3,ribbonlabel+"(slow)");

   return(0);
}

string getNtoTF(int tf)
{
   string stf="-";
   if (tf==1)
      stf="M1";
   else if (tf==5)
      stf="M5";
   else if (tf==15)
      stf="M15";
   else if (tf==30)
      stf="M30";
   else if (tf==60)
      stf="H1";
   else if (tf==240)
      stf="H4";
   else if (tf==1440)
      stf="D1";
   return(stf);
}

void setConstantColour(int tf)
{
   if (tf==5)
      ribbonColor=level0color;
   else if (tf==15)
      ribbonColor=level1color;
   else if (tf==30)
      ribbonColor=level2color;
   else if (tf==60)
      ribbonColor=level3color;
   else if (tf==240)
      ribbonColor=level4color;
   else if (tf==1440)
      ribbonColor=level5color;
   
}
int getNextTF(int tf)
{
   int nextTF=5;
   if (tf==5)
   {
      nextTF=15;
      if (ribbonColoursConstant)
         ribbonColor=level1color;
   }
   else if (tf==15)
   {
      nextTF=30;
      if (ribbonColoursConstant)
         ribbonColor=level2color;
   }
   else if (tf==30)
   {
      nextTF=60;
      if (ribbonColoursConstant)
         ribbonColor=level3color;
   }
   else if (tf==60)
   {
      nextTF=240;
      if (ribbonColoursConstant)
         ribbonColor=level4color;
   }
   else if (tf==240)
   {
      nextTF=1440;   // D1
      fastColor=Yellow;
      if (ribbonColoursConstant)
         ribbonColor=level5color;
   }
   else if (tf==1440)
   {
      nextTF=10080;  // W1
      fastColor=Yellow;
      screenDoor=True;
   }
   else
      nextTF=0;
   return nextTF;
}

int getPrevTF(int tf)
{
   int prevTF=5;
   if (tf==5)
      prevTF=1;
   else if (tf==15)
      prevTF=5;
   else if (tf==30)
      prevTF=15;
   else if (tf==60)
      prevTF=30;
   else if (tf==240)
      prevTF=60;
   else if (tf==1440)
      prevTF=240;
   return prevTF;
}

void writeText(int unique, string txt, double Lprice, color Lcolour)
{
   string label=prefix+(string)unique+txt;
   ObjectDelete(label);
   ObjectCreate(label, OBJ_TEXT, 0, 0, 0);
   ObjectSetText(label,txt,labelFontSize, labelFont, Lcolour);
   ObjectMove(label,0, Time[0]+6*60*Period(),Lprice);
}

int deinit()
{
   for(int i = ObjectsTotal() - 1; i >= 0; i--)
   {
      string label = ObjectName(i);
      if(StringSubstr(label, 0, StringLen(prefix)) != prefix )
         continue;
      ObjectDelete(label);   
   }
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//

int start()
{
   int counted_bars=IndicatorCounted();
   int limit,i;
   double Labelprice;
   
   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=Bars-counted_bars;

   if (MA1Period==0) return(0); //out of range

//   Alert("cb="+counted_bars+" limit="+limit+" max="+NbOfCandles);
   limit = MathMin(limit, NbOfCandles);
   for(i=0; i<limit; i++)  
   {
      buffer1[i] = iCustom(NULL,TimeFrame,"T3MA",MA1Period,factor1,0,i);
      buffer2[i] = iCustom(NULL,TimeFrame,"T3MA",MA2Period,factor2,0,i);
      buffer3[i] = buffer1[i];
      buffer4[i] = buffer2[i];
      if (i==0 && showRibbonLabels)
      {
         if (buffer1[i]>buffer2[i])
            Labelprice=buffer2[i]+(buffer1[i]-buffer2[i])/2;
         else
            Labelprice=buffer1[i]+(buffer2[i]-buffer1[i])/2;
         writeText(MA2Period-1,"<<<@@@@@@",Labelprice,ribbonColor);         
         writeText(MA2Period,"---"+ribbonlabel,Labelprice,FontColour);         
      }
   }
   return(0);
}

