//+------------------------------------------------------------------+
//|                                     BetterVolume1.4 [ay-mod].mq4 |
//|                          CopyLeft  2011, ahmad.yani@hotmail.com |
//|                                          http://eyesfx.web44.net |
//+------------------------------------------------------------------+
#property copyright "CopyLeft  2011, ahmad.yani@hotmail.com"
#property link      "http://eyesfx.web44.net"

//------
/*
http://EminiWatch.com
You can figure out whether the Professionals are buying or selling by analyzing: 
? Volume transacted at the bid or the ask 
? High to low range of the bar, and 
? Average trade size. 
The Better Volume indicator improves on your typical volume histogram by coloring the bars based on 5 criteria: 
? Volume Climax Up  high volume, high range, up bars (red) 
? Volume Climax Down  high volume, high range, down bars (white) 
? High Volume Churn  high volume, low range bars (green, PaintBar blue) 
? Volume Climax plus High Volume Churn  both the above conditions (magenta) 
? Low Volume  low volume bars (yellow) 
? When there are no volume signals the default histogram bar coloring is cyan 
*/

/*
  BetterVolume1.4 [ay.mod]
- rewrite from the original BetterVolume1.4's code by thatwasme (forexfactory.com user, credit is belong to him, 
  Now I or maybe You can easily reuse a main function "identifyVolume(int shift, int& iResult, double& avgvol)"
- adding some features: Popup Alert, Email Alert, Histogram Coloring, Histogram Width, GreaterThanPrev2, LessThanPrev2
*/

#property indicator_separate_window
#property indicator_buffers 8

#property indicator_minimum 0

#define IND_NAME  "iBetterVolume"

extern int     NumBars              = 1000;
extern int     LookBack             = 20;
extern color   ClrClimaxHigh        = Red;
extern color   ClrClimaxLow         = White;
extern color   ClrHighChurn         = DarkGreen;
extern color   ClrClimaxChurn       = Magenta;
extern color   ClrLowestVol         = Yellow;
extern color   ClrGreaterThanPrev2  = LimeGreen;
extern color   ClrLessThanPrev2     = IndianRed;
extern color   ClrNeutralVol        = DodgerBlue;
extern int     HistoWidth           = 2;
extern string  Info                 = "Alert for Climax High/Low/Churn and High Volume Churn";
extern bool    EnablePopupAlert     = false;
extern bool    EnableEmailAlert     = false;
extern bool    ShowLegend           = false;

//--buffers
double     BuffClimaxHigh[]
         , BuffClimaxLow[]
         , BuffHighChurn[]
         , BuffClimaxChurn[]
         , BuffLowestVol[]
         , BuffHigherThan2Prev[]
         , BuffLowerThan2Prev[]
         , BuffNeutralVol[]
         //, BuffAverageVol[]
         ;
//--global vars         
color      gcaBuffColor[8];
string     gsaBuffLabel[8] = 
           {
               "Volume Climax Up"
             , "Volume Climax Down"
             , "High Volume Churn"
             , "Climax Churn"
             , "Lowest Volume"
             , "Greater Than Prev 2"
             , "Less Than Prev 2"
             , "Neutral Volume"           
           }
         , gsPref = "ibv."
         ;

         
//#include <ay-iBetterVolume.mqh>  
#include <ay-iBetterVolume.v1.10.mqh>                         

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   int i, hwidth;   
   
   IndicatorShortName(IND_NAME);
   IndicatorDigits(0);   
   
   gcaBuffColor[IDX_CH]  = ClrClimaxHigh;
   gcaBuffColor[IDX_CL]  = ClrClimaxLow;
   gcaBuffColor[IDX_HC]  = ClrHighChurn;
   gcaBuffColor[IDX_CC]  = ClrClimaxChurn;
   gcaBuffColor[IDX_LV]  = ClrLowestVol;
   gcaBuffColor[IDX_H2P] = ClrGreaterThanPrev2;
   gcaBuffColor[IDX_L2P] = ClrLessThanPrev2;
   gcaBuffColor[IDX_NV]  = ClrNeutralVol;
   
   SetIndexBuffer(IDX_CH,  BuffClimaxHigh);
   SetIndexBuffer(IDX_CL,  BuffClimaxLow);
   SetIndexBuffer(IDX_HC,  BuffHighChurn);
   SetIndexBuffer(IDX_CC,  BuffClimaxChurn);
   SetIndexBuffer(IDX_LV,  BuffLowestVol);
   SetIndexBuffer(IDX_H2P, BuffHigherThan2Prev);
   SetIndexBuffer(IDX_L2P, BuffLowerThan2Prev);
   SetIndexBuffer(IDX_NV,  BuffNeutralVol);

   
   for (i=0; i<=IDX_NV; i++)
   {            
      SetIndexStyle(i, DRAW_HISTOGRAM, EMPTY, HistoWidth, gcaBuffColor[i]);
      SetIndexDrawBegin(i, LookBack);
      SetIndexLabel(i, gsaBuffLabel[i]);         
   }
     
   
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
   string name;
   for(int cnt=ObjectsTotal()-1;cnt>=0;cnt--)
   {
      name=ObjectName(cnt);
      if (StringSubstr(name,0,StringLen(gsPref)) == gsPref)       
         ObjectDelete(name); 
   }   
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   int      counted_bars = IndicatorCounted();
   int      iBar, iLimit, iBarsToRecalc, iResult;
   double   dBarVol, dAvgVol;
   string   sMsg, sSubject;
   
   static bool bFirstRun = true;
   
//---- check for possible errors
   if(counted_bars<0) return(-1);
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   
   if(Bars < LookBack) return(0);   
   
   iBarsToRecalc = Bars-counted_bars;
   iLimit        = MathMin(NumBars-1, iBarsToRecalc);
   
   if (bFirstRun) sendAlert("","", true); 
   
   for(iBar=0; iBar<iLimit; iBar++)
   {
      dBarVol = Volume[iBar];
      
      setEmptyBuffer(iBar);
      
      iResult = identifyVolume(Period(), iBar, LookBack);
      
      switch (iResult)
      {
         case IDX_CH  : BuffClimaxHigh      [iBar] = dBarVol; break;
         case IDX_CL  : BuffClimaxLow       [iBar] = dBarVol; break;
         case IDX_HC  : BuffHighChurn       [iBar] = dBarVol; break;
         case IDX_CC  : BuffClimaxChurn     [iBar] = dBarVol; break;
         case IDX_LV  : BuffLowestVol       [iBar] = dBarVol; break;
         case IDX_H2P : BuffHigherThan2Prev [iBar] = dBarVol; break;
         case IDX_L2P : BuffLowerThan2Prev  [iBar] = dBarVol; break;
         case IDX_NV  : BuffNeutralVol      [iBar] = dBarVol; break;       
      }      
            
      //-->Alert            
      int iAlert=-1; 
      
      if (iBar!=0) continue;
      
      if ( BuffClimaxHigh  [iBar+1] != EMPTY_VALUE ) iAlert = IDX_CH;
      if ( BuffClimaxLow   [iBar+1] != EMPTY_VALUE ) iAlert = IDX_CL;
      if ( BuffHighChurn   [iBar+1] != EMPTY_VALUE ) iAlert = IDX_HC;
      if ( BuffClimaxChurn [iBar+1] != EMPTY_VALUE ) iAlert = IDX_CC;      
                  
      if (    (EnablePopupAlert || EnableEmailAlert)                  
           && (iAlert != -1)                         
           && isNewBar()           
           && !bFirstRun
         )
      {
      
         sSubject = StringConcatenate( "BV Alert> ", Symbol(), ",", Period(), " Mnts> ", gsaBuffLabel[iAlert] );
         sMsg     = StringConcatenate
                     (
                          sSubject
                        , "\r\nCurrent Price: ", DoubleToStr(Close[0], Digits)
                        , "\r\nBroker Time: "  , TimeToStr(TimeCurrent())
                        , "\r\nLocal Time: "   , TimeToStr(TimeLocal()) 
                        , "\r\n.\r\n"          
                     );
                  
         sendAlert(sSubject, sMsg);
      }
      //<--Alert
                            
   }
   
   
   if (bFirstRun) 
   {         
      if (ShowLegend) showLegend();
      bFirstRun = false; 
   }
   
   
   return(0);
}

void setEmptyBuffer(int shift)
{
   BuffClimaxHigh      [shift] = EMPTY_VALUE;
   BuffClimaxLow       [shift] = EMPTY_VALUE;
   BuffHighChurn       [shift] = EMPTY_VALUE;
   BuffClimaxChurn     [shift] = EMPTY_VALUE;
   BuffLowestVol       [shift] = EMPTY_VALUE;
   BuffHigherThan2Prev [shift] = EMPTY_VALUE;
   BuffLowerThan2Prev  [shift] = EMPTY_VALUE;
   BuffNeutralVol      [shift] = EMPTY_VALUE;
}

void sendAlert(string subject, string msg,  bool init=false)
{
   static datetime dtLastAlert;
   static int      iAlert = IDX_NV;  
   
   if (init) { dtLastAlert = Time[0]; return; }   

   if (dtLastAlert != Time[0])
   {
      dtLastAlert = Time[0];
      
      if (EnablePopupAlert) Alert(msg);
      
      if (EnableEmailAlert) SendMail(subject, msg);                  
   }
   
}

bool isNewBar()
{
   static datetime lastbartime;
   datetime currbartime = Time[0];

   if(lastbartime != currbartime)
   {
      lastbartime = currbartime;
      return (true);
   }
   return(false);     
}

void showLegend()
{
   int   i, x=5, y=1
       , winid = WindowFind(IND_NAME)
       ;
   
   lblCreate(
         "legend"
       , x, y
       , "Legend :", 1
       , gcaBuffColor[IDX_CL], 9
       , "Tahoma Bold", winid
       );
   
   y+=15;
   
   for (i=0; i<ArraySize(gsaBuffLabel); i++)
   {
      lblCreate(
           strReplace(gsaBuffLabel[i]," ", "")
         , x, y
         , gsaBuffLabel[i], 1
         , gcaBuffColor[i], 8
         , "Tahoma", winid
         );
      y+=12;
   }
}

void lblCreate(string name,int x,int y, string text, int corner =0
   , color c=Silver, int size=8, string font="Tahoma", int winid=0)
{
   
   name = gsPref + name;
   if (ObjectFind(name) != winid)
      ObjectCreate (name,OBJ_LABEL,winid,0,0);
      
   ObjectSet    (name,OBJPROP_CORNER,corner);
   ObjectSet    (name,OBJPROP_XDISTANCE,x);
   ObjectSet    (name,OBJPROP_YDISTANCE,y);
   ObjectSetText(name,text,size,font,c);
} 

string strReplace(string str, string csearch, string creplace)
{
   int pos, loop;

   for(;;)
   {
      pos = StringFind(str, csearch, 0);      
      if (pos == -1) return(str);
      
      str = StringConcatenate
       ( 
         StringSubstr(str, 0, pos)
         , creplace
         , StringSubstr(str, pos+StringLen(csearch), StringLen(str)-pos)
       );      
       
      //--safety
      if (loop>StringLen(str)) return(str);
      loop++;
   }

}
//+------------------------------------------------------------------+